from __future__ import unicode_literals

from functools import wraps

from django.conf import settings
from django.contrib.auth.views import redirect_to_login as dj_redirect_to_login
from django.core.exceptions import PermissionDenied

from rolepermissions.checkers import has_role, has_permission
from rolepermissions.utils import user_is_authenticated


def has_role_decorator(role, redirect_to_login=None):
    def request_decorator(dispatch):
        @wraps(dispatch)
        def wrapper(request, *args, **kwargs):
            user = request.user
            if user_is_authenticated(user):
                if has_role(user, role):
                    return dispatch(request, *args, **kwargs)

            redirect = redirect_to_login
            if redirect is None:
                redirect = getattr(
                    settings, 'ROLEPERMISSIONS_REDIRECT_TO_LOGIN', False)
            if redirect:
                return dj_redirect_to_login(request.get_full_path())
            raise PermissionDenied
        return wrapper
    return request_decorator


def has_permission_decorator(permission_name, redirect_to_login=None):
    def request_decorator(dispatch):
        @wraps(dispatch)
        def wrapper(request, *args, **kwargs):
            user = request.user
            if user_is_authenticated(user):
                if has_permission(user, permission_name):
                    return dispatch(request, *args, **kwargs)

            redirect = redirect_to_login
            if redirect is None:
                redirect = getattr(
                    settings, 'ROLEPERMISSIONS_REDIRECT_TO_LOGIN', False)
            if redirect:
                return dj_redirect_to_login(request.get_full_path())
            raise PermissionDenied
        return wrapper
    return request_decorator




from django.test import TestCase
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from model_mommy import mommy

from rolepermissions.roles import RolesManager, AbstractUserRole
from rolepermissions.roles import (
    get_user_roles, retrieve_role,
    assign_role, remove_role, clear_roles
)
from rolepermissions.permissions import (
    grant_permission, revoke_permission,
    available_perm_status, available_perm_names)
from rolepermissions.checkers import has_permission
from rolepermissions.exceptions import (
    RoleDoesNotExist, RolePermissionScopeException)


class ShoRole1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }


class ShoRole2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }


class ShoRole3(AbstractUserRole):
    role_name = 'sho_new_name'
    available_permissions = {
        'permission5': False,
        'permission6': False,
    }


class ShoRole4(AbstractUserRole):
    available_permissions = {
        'permission1': False,
        'permission3': False,
    }


class AssignRoleTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())

    def test_assign_role(self):
        user = self.user

        assign_role(user, 'sho_role1')

        self.assertListEqual([ShoRole1], get_user_roles(user))

    def test_assign_role_by_class(self):
        user = self.user

        assign_role(user, ShoRole1)

        self.assertListEqual([ShoRole1], get_user_roles(user))

    def test_assign_invalid_role(self):
        user = self.user

        with self.assertRaises(RoleDoesNotExist):
            assign_role(user, 'no role')

    def test_assign_multiple_roles(self):
        user = self.user

        assign_role(user, ShoRole1)
        assign_role(user, ShoRole2)

        self.assertListEqual([ShoRole1, ShoRole2], get_user_roles(user))


class RemoveRoleTests(TestCase):
    enter_surgery_room = "enter_surgery_room"
    operate = "operate"

    class Doctor(AbstractUserRole):
        available_permissions = {
            "enter_surgery_room": False,
        }

    class Surgeon(AbstractUserRole):
        available_permissions = {
            "enter_surgery_room": True,
            "operate": True,
        }

    class Anesthesiologist(AbstractUserRole):
        available_permissions = {
            "enter_surgery_room": True,
            "operate": False,
        }

    def setUp(self):
        self.user = mommy.make(get_user_model())

    def test_remove_role_from_user(self):
        assign_role(self.user, self.Doctor)
        remove_role(self.user, self.Doctor)

        self.assertListEqual([], get_user_roles(self.user))

    def test_remove_role_from_user_with_multiple_roles(self):
        """Ensure that remove_role() only removes the role specified, not all of the user's roles."""
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)
        assign_role(self.user, self.Anesthesiologist)

        remove_role(self.user, self.Doctor)

        self.assertListEqual([self.Anesthesiologist, self.Surgeon], get_user_roles(self.user))

    def test_remove_role_user_isnt_assigned_to(self):
        remove_role(self.user, self.Doctor)

        self.assertListEqual([], get_user_roles(self.user))

    def test_remove_invalid_role(self):
        with self.assertRaises(RoleDoesNotExist):
            assign_role(self.user, 'no role')

    def test_remove_role_reinstates_permissions_correctly_scenario_1(self):
        """
        Initial Roles:
            Doctor
            Surgeon

        Actions:
            Remove role: Surgeon

        Expected resulting permissions:
            enter_surgery_room = False
            operate = False
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)

        remove_role(self.user, self.Surgeon)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertFalse(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_2(self):
        """
        Initial Roles:
            Doctor
            Surgeon

        Actions:
            Remove role: Doctor

        Expected resulting permission:
            enter_surgery_room = True
            operate = True
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)

        remove_role(self.user, self.Doctor)

        self.assertTrue(has_permission(self.user, self.enter_surgery_room))
        self.assertTrue(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_3(self):
        """
        Initial Roles:
            Doctor
            Surgeon

        Actions:
            Revoke permission: enter_surgery_room
            Remove role: Surgeon

        Expected resulting permission:
            enter_surgery_room = False
            operate = False
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)

        revoke_permission(self.user, self.enter_surgery_room)
        remove_role(self.user, self.Surgeon)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertFalse(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_4(self):
        """
        Initial Roles:
            Doctor
            Surgeon

        Actions:
            Revoke permission: enter_surgery_room
            Remove role: Doctor

        Expected resulting permission:
            enter_surgery_room = False
            operate = True
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)

        revoke_permission(self.user, self.enter_surgery_room)
        remove_role(self.user, self.Doctor)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertTrue(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_5(self):
        """
        Initial Roles:
            Doctor
            Surgeon

        Actions:
            Grant permission: operate
            Remove role: Surgeon

        Expected resulting permission:
            enter_surgery_room = False
            operate = True
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)

        grant_permission(self.user, self.operate)
        remove_role(self.user, self.Surgeon)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertFalse(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_6(self):
        """
        Initial Roles:
            Doctor
            Surgeon

        Actions:
            Revoke permission: enter_surgery_room
            Revoke permission: operate
            Remove role: Doctor

        Expected resulting permission:
            enter_surgery_room = False
            operate = False
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)

        revoke_permission(self.user, self.enter_surgery_room)
        revoke_permission(self.user, self.operate)
        remove_role(self.user, self.Doctor)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertFalse(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_7(self):
        """
        Initial Roles:
            Doctor
            Surgeon
            Anesthesiologist

        Actions:
            Remove role: Surgeon

        Expected resulting permission:
            enter_surgery_room = True
            operate = False
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)
        assign_role(self.user, self.Anesthesiologist)

        remove_role(self.user, self.Surgeon)

        self.assertTrue(has_permission(self.user, self.enter_surgery_room))
        self.assertFalse(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_8(self):
        """
        Initial Roles:
            Doctor
            Surgeon
            Anesthesiologist

        Actions:
            Remove role: Doctor

        Expected resulting permission:
            enter_surgery_room = True
            operate = True
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)
        assign_role(self.user, self.Anesthesiologist)

        remove_role(self.user, self.Doctor)

        self.assertTrue(has_permission(self.user, self.enter_surgery_room))
        self.assertTrue(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_9(self):
        """
        Initial Roles:
            Doctor
            Surgeon
            Anesthesiologist

        Actions:
            Revoke permission: enter_surgery_room
            Remove role: Surgeon

        Expected resulting permission:
            enter_surgery_room = False
            operate = False
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)
        assign_role(self.user, self.Anesthesiologist)

        revoke_permission(self.user, self.enter_surgery_room)
        remove_role(self.user, self.Surgeon)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertFalse(has_permission(self.user, self.operate))

    def test_remove_role_reinstates_permissions_correctly_scenario_10(self):
        """
        Initial Roles:
            Doctor
            Surgeon
            Anesthesiologist

        Actions:
            Revoke permission: enter_surgery_room
            Remove role: Doctor

        Expected resulting permission:
            enter_surgery_room = False
            operate = True
        """
        assign_role(self.user, self.Doctor)
        assign_role(self.user, self.Surgeon)
        assign_role(self.user, self.Anesthesiologist)

        revoke_permission(self.user, self.enter_surgery_room)
        remove_role(self.user, self.Doctor)

        self.assertFalse(has_permission(self.user, self.enter_surgery_room))
        self.assertTrue(has_permission(self.user, self.operate))


class ClearRolesTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())

    def test_clear_roles(self):
        user = self.user

        assign_role(user, ShoRole1)
        assign_role(user, ShoRole2)
        assign_role(user, ShoRole3)

        clear_roles(user)

        self.assertListEqual([], get_user_roles(user))


class GetUserRoleTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())

    def test_returns_list(self) :
        user = self.user

        user_roles = get_user_roles(user)
        self.assertEquals(type(user_roles), type([]))

    def test_get_user_roles(self):
        user = self.user

        ShoRole1.assign_role_to_user(user)

        self.assertListEqual([ShoRole1], get_user_roles(user))

    def test_get_user_roles_multiple_roles(self):
        user = self.user

        ShoRole1.assign_role_to_user(user)
        ShoRole3.assign_role_to_user(user)

        self.assertListEqual([ShoRole3, ShoRole1], get_user_roles(user))

    def test_user_without_role(self):
        user = self.user

        self.assertListEqual([], get_user_roles(user))

    def test_ensure_user_roles_are_in_order(self):
        user = self.user

        assign_role(user, ShoRole2)
        assign_role(user, ShoRole4)
        assign_role(user, ShoRole1)
        assign_role(user, ShoRole3)

        self.assertListEqual([ShoRole3, ShoRole1, ShoRole2, ShoRole4], get_user_roles(user))

    def test_dont_return_non_role_groups(self):
        user = self.user

        assign_role(user, ShoRole1)
        assign_role(user, ShoRole2)
        assign_role(user, ShoRole3)
        assign_role(user, ShoRole4)

        other_group = mommy.make(Group)
        user.groups.add(other_group)

        self.assertNotIn(other_group, get_user_roles(user))

    def test_queries_no_prefetch(self):
        user = self.user

        assign_role(user, ShoRole1)
        assign_role(user, ShoRole2)
        assign_role(user, ShoRole3)

        fetched_user = get_user_model().objects.get(pk=self.user.pk)
        N = 3
        with self.assertNumQueries(N):  # One query (fetch roles) per call
            for i in range(N):
                user_roles = get_user_roles(fetched_user)

    def test_queries_with_prefetch(self):
        user = self.user

        assign_role(user, ShoRole1)
        assign_role(user, ShoRole2)
        assign_role(user, ShoRole3)

        fetched_user = get_user_model().objects.prefetch_related('groups').get(pk=self.user.pk)
        N = 3
        with self.assertNumQueries(0):  # all data required is cached with fetched_user
            for i in range(N):
                user_roles = get_user_roles(fetched_user)

    def tearDown(self):
        RolesManager._roles = {}


class AvailablePermStatusTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())
        self.user_role = ShoRole2.assign_role_to_user(self.user)

    def test_permission_hash(self):
        perm_hash = available_perm_status(self.user)

        self.assertTrue(perm_hash['permission3'])
        self.assertFalse(perm_hash['permission4'])

    def test_permission_hash_multiple_groups(self):
        """
        If a user has a permission in any role, that permission should show as True,
        no matter what other roles dictate.
        """
        ShoRole1.assign_role_to_user(self.user)
        ShoRole4.assign_role_to_user(self.user)

        perm_hash = available_perm_status(self.user)

        self.assertTrue(perm_hash['permission1'])
        self.assertTrue(perm_hash['permission2'])
        self.assertTrue(perm_hash['permission3'])
        self.assertFalse(perm_hash['permission4'])

    def test_permission_hash_after_modification(self):
        revoke_permission(self.user, 'permission3')

        perm_hash = available_perm_status(self.user)

        self.assertFalse(perm_hash['permission3'])
        self.assertFalse(perm_hash['permission4'])


class AvailablePermNamesTests(TestCase):

    def assert_available_perm_names_equals_available_perm_status(self):
        perm_hash = available_perm_status(self.user)
        perm_names = available_perm_names(self.user)

        self.assertEqual( set(perm_names), set(p for p, has_perm in perm_hash.items() if has_perm) )

    def setUp(self):
        self.user = mommy.make(get_user_model())
        self.user_role = ShoRole2.assign_role_to_user(self.user)

    def test_permission_names(self):
        self.assert_available_perm_names_equals_available_perm_status()

    def test_permission_names_multiple_groups(self):
        """
        If a user has a permission in any role, that permission should show as True,
        no matter what other roles dictate.
        """
        ShoRole1.assign_role_to_user(self.user)
        ShoRole4.assign_role_to_user(self.user)

        self.assert_available_perm_names_equals_available_perm_status()

    def test_permission_names_after_modification(self):
        revoke_permission(self.user, 'permission3')

        self.assert_available_perm_names_equals_available_perm_status()

    def test_queries_no_prefetch(self):
        fetched_user = get_user_model().objects.get(pk=self.user.pk)
        N = 3
        with self.assertNumQueries(2 * N):  # Two query (fetch roles, fetch permissions) per call
            for i in range(N):
                perm_names = available_perm_names(fetched_user)

    def test_queries_with_prefetch(self):
        fetched_user = get_user_model().objects.prefetch_related('groups', 'user_permissions').get(pk=self.user.pk)
        N = 3
        with self.assertNumQueries(0):  # all data required is cached with fetched_user
            for i in range(N):
                perm_names = available_perm_names(fetched_user)


class GrantPermissionTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())
        self.user_role = ShoRole2.assign_role_to_user(self.user)

    def test_grant_permission(self):
        user = self.user

        grant_permission(user, 'permission4')

        self.assertTrue(has_permission(user, 'permission4'))

    def test_grat_granted_permission(self):
        user = self.user

        grant_permission(user, 'permission3')

        self.assertTrue(has_permission(user, 'permission3'))

    def test_not_allowed_permission(self):
        user = self.user

        with self.assertRaises(RolePermissionScopeException):
            grant_permission(user, 'permission1')

    def test_not_allowed_permission_multiple_roles(self):
        user = self.user
        ShoRole3.assign_role_to_user(self.user)

        with self.assertRaises(RolePermissionScopeException):
            grant_permission(user, 'permission1')


class RevokePermissionTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())
        self.user_role = ShoRole2.assign_role_to_user(self.user)

    def test_revoke_permission(self):
        user = self.user

        revoke_permission(user, 'permission3')

        self.assertFalse(has_permission(user, 'permission3'))

    def test_revoke_revoked_permission(self):
        user = self.user

        revoke_permission(user, 'permission4')

        self.assertFalse(has_permission(user, 'permission4'))

    def test_not_allowed_permission(self):
        user = self.user

        with self.assertRaises(RolePermissionScopeException):
            revoke_permission(user, 'permission1')

    def test_not_allowed_permission_multiple_roles(self):
        user = self.user
        ShoRole3.assign_role_to_user(self.user)

        with self.assertRaises(RolePermissionScopeException):
            revoke_permission(user, 'permission1')


class RetrieveRole(TestCase):

    def setUp(self):
        pass

    def test_retrive_role1(self):
        self.assertEquals(retrieve_role('sho_role1'), ShoRole1)

    def test_retrive_role2(self):
        self.assertEquals(retrieve_role('sho_role2'), ShoRole2)

    def test_retrive_role3(self):
        self.assertEquals(retrieve_role('sho_new_name'), ShoRole3)

    def test_retrieve_unknowun_role(self):
        role = retrieve_role('unknowun_role')
        self.assertIsNone(role)



from __future__ import unicode_literals

import re
import collections


def user_is_authenticated(user):
    if isinstance(user.is_authenticated, collections.Callable):
        authenticated = user.is_authenticated()
    else:
        authenticated = user.is_authenticated

    return authenticated


def camelToSnake(s):
    """
    https://gist.github.com/jaytaylor/3660565
    Is it ironic that this function is written in camel case, yet it
    converts to snake case? hmm..
    """
    _underscorer1 = re.compile(r'(.)([A-Z][a-z]+)')
    _underscorer2 = re.compile('([a-z0-9])([A-Z])')

    subbed = _underscorer1.sub(r'\1_\2', s)
    return _underscorer2.sub(r'\1_\2', subbed).lower()


def snake_to_title(s):
    return ' '.join(x.capitalize() for x in s.split('_'))


def camel_or_snake_to_title(s):
    return snake_to_title(camelToSnake(s))



from django.conf import settings
from django.contrib import admin, auth
from django.contrib.auth.models import Group
from django.contrib.auth.admin import UserAdmin
from rolepermissions import roles

ROLEPERMISSIONS_REGISTER_ADMIN = getattr(settings, 'ROLEPERMISSIONS_REGISTER_ADMIN', False)
UserModel = auth.get_user_model()


class RolePermissionsUserAdminMixin(object):
    """ Must be mixed in with an UserAdmin class"""
    def save_related(self, request, form, formsets, change):
        user = UserModel.objects.get(pk=form.instance.pk)
        old_user_roles = set(r.get_name() for r in roles.get_user_roles(user))
        super(RolePermissionsUserAdminMixin, self).save_related(request, form, formsets, change)

        new_user_groups = set(g.name for g in user.groups.all())

        for role_name in (old_user_roles - new_user_groups):  # roles removed from User's groups
            try:  # put the recently removed group back, let rolepermissions remove it...
                group = Group.objects.get(name=role_name)
                user.groups.add(group)
            except Group.DoesNotExist:
                pass
            roles.remove_role(user, role_name)

        for group_name in (new_user_groups - old_user_roles):  # groups potentially added to User's roles
            try:
                roles.assign_role(user, group_name)
            except roles.RoleDoesNotExist:
                pass


class RolePermissionsUserAdmin(RolePermissionsUserAdminMixin, UserAdmin):
    pass


if ROLEPERMISSIONS_REGISTER_ADMIN:
    admin.site.unregister(UserModel)
    admin.site.register(UserModel, RolePermissionsUserAdmin)




from django.test import TestCase
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from model_mommy import mommy

from rolepermissions.roles import RolesManager, AbstractUserRole, get_or_create_permission


class RolRole1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }


class RolRole2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }


class RolRole3(AbstractUserRole):
    role_name = 'new_name'
    available_permissions = {
        'permission5': False,
        'permission6': False,
    }

class RolRole4(AbstractUserRole):
    available_permissions = {
        'permission_number_7': True,
        'PermissionNumber8': True,
    }

class AbstractUserRoleTests(TestCase):

    def setUp(self):
        pass

    def test_get_name(self):
        self.assertEquals(RolRole1.get_name(), 'rol_role1')
        self.assertEquals(RolRole2.get_name(), 'rol_role2')
        self.assertEquals(RolRole3.get_name(), 'new_name')

    def test_assign_Role1_default_permissions(self):
        user = mommy.make(get_user_model())

        RolRole1.assign_role_to_user(user)
        permissions = user.user_permissions.all()

        permission_names_list = [perm.codename for perm in permissions]

        self.assertIn('permission1', permission_names_list)
        self.assertIn('permission2', permission_names_list)
        self.assertEquals(len(permissions), 2)

    def test_assign_Role2_default_permissions(self):
        user = mommy.make(get_user_model())

        RolRole2.assign_role_to_user(user)
        permissions = user.user_permissions.all()

        permission_names_list = [perm.codename for perm in permissions]

        self.assertIn('permission3', permission_names_list)
        self.assertNotIn('permission4', permission_names_list)
        self.assertEquals(len(permissions), 1)

    def test_assign_Role3_default_permissions(self):
        user = mommy.make(get_user_model())

        RolRole3.assign_role_to_user(user)
        permissions = user.user_permissions.all()

        permission_names_list = [perm.codename for perm in permissions]

        self.assertNotIn('permission5', permission_names_list)
        self.assertNotIn('permission6', permission_names_list)
        self.assertEquals(len(permissions), 0)

    def test_assign_role_to_user(self):
        user = mommy.make(get_user_model())

        user_role = RolRole1.assign_role_to_user(user)

        self.assertEquals(user_role.name, 'rol_role1')

    def test_instanciate_role(self):
        user = mommy.make(get_user_model())

        user_role = RolRole1.assign_role_to_user(user)

        self.assertIsNotNone(user_role.pk)

    def test_assign_multiple_roles(self):
        user = mommy.make(get_user_model())

        user_role_1 = RolRole1.assign_role_to_user(user)
        self.assertEquals(user_role_1.name, 'rol_role1')

        user_role_2 = RolRole2.assign_role_to_user(user)
        self.assertEquals(user_role_2.name, 'rol_role2')

        user_groups = user.groups.all()

        self.assertEqual(2, len(user_groups))
        self.assertIn(user_role_1, user_groups)
        self.assertIn(user_role_2, user_groups)

    def test_dont_remove_other_groups(self):
        user = mommy.make(get_user_model())
        other_group = mommy.make(Group)
        user.groups.add(other_group)

        user_role_1 = RolRole1.assign_role_to_user(user)
        self.assertEquals(user_role_1.name, 'rol_role1')

        user_role_2 = RolRole2.assign_role_to_user(user)
        self.assertEquals(user_role_2.name, 'rol_role2')

        RolRole2.remove_role_from_user(user)

        user_groups = user.groups.all()

        self.assertIn(user_role_1, user_groups)
        self.assertIn(other_group, user_groups)

    def test_permission_names_list(self):
        self.assertIn('permission1', RolRole1.permission_names_list())
        self.assertIn('permission2', RolRole1.permission_names_list())

        self.assertIn('permission3', RolRole2.permission_names_list())
        self.assertIn('permission4', RolRole2.permission_names_list())

    def test_permission_labels(self):
        user = mommy.make(get_user_model())

        RolRole4.assign_role_to_user(user)
        permissions = user.user_permissions.all()

        permission_labels = [perm.name for perm in permissions]

        self.assertIn('Permission Number 7', permission_labels)
        self.assertIn('Permission Number8', permission_labels)
        self.assertEquals(len(permissions), 2)


class RolesManagerTests(TestCase):

    def setUp(self):
        pass

    def test_retrieve_role(self):
        self.assertEquals(RolesManager.retrieve_role('rol_role1'), RolRole1)
        self.assertEquals(RolesManager.retrieve_role('rol_role2'), RolRole2)


class GetOrCreatePermissionsTests(TestCase):

    def setUp(self):
        pass

    def test_create_default_named_permission(self):
        perm_snake, _created = get_or_create_permission("my_perm_name1")
        self.assertEqual(perm_snake.name, "My Perm Name1")

        perm_camel, _created = get_or_create_permission("myPermName2")
        self.assertEqual(perm_camel.name, "My Perm Name2")

    def test_create_and_get_named_permission(self) :
        perm1, _created = get_or_create_permission("my_perm_name", name="My Custom Name")
        self.assertEqual(perm1.name, "My Custom Name")

        perm2, _created = get_or_create_permission("my_perm_name", name="My Custom Name")
        self.assertEqual(perm1, perm2)

    def test_create_and_get_specialty_named_permission(self) :
        def name_perm(codename):
            return "Custom-"+codename
        perm, _created = get_or_create_permission("my_perm_name", name_perm)
        self.assertEqual(perm.name, "Custom-my_perm_name")

    def test_backwards_compat_with_unnamed_permission(self) :
        unnamed_perm, _created = get_or_create_permission("my_perm_name", name="")
        self.assertEqual(unnamed_perm.name, "")

        perm, _created = get_or_create_permission("my_perm_name")
        self.assertEqual(unnamed_perm, perm)



from __future__ import unicode_literals

import inspect

from six import add_metaclass

from django.contrib.auth.models import Group, Permission
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from rolepermissions.utils import camelToSnake, camel_or_snake_to_title
from rolepermissions.exceptions import RoleDoesNotExist


registered_roles = {}


class RolesManager(object):

    def __iter__(cls):
        return iter(registered_roles)

    @classmethod
    def retrieve_role(cls, role_name):
        if role_name in registered_roles:
            return registered_roles[role_name]

    @classmethod
    def get_roles_names(cls):
        return registered_roles.keys()

    @classmethod
    def get_roles(cls):
        return registered_roles.values()


class RolesClassRegister(type):

    def __new__(cls, name, parents, dct):
        role_class = super(RolesClassRegister, cls).__new__(cls, name, parents, dct)
        if object not in parents:
            registered_roles[role_class.get_name()] = role_class
        return role_class


@add_metaclass(RolesClassRegister)
class AbstractUserRole(object):

    @classmethod
    def get_name(cls):
        if hasattr(cls, 'role_name'):
            return cls.role_name

        return camelToSnake(cls.__name__)

    @classmethod
    def assign_role_to_user(cls, user):
        """
        Assign this role to a user.

        :returns: :py:class:`django.contrib.auth.models.Group` The group for the
            new role.
        """
        group, _created = Group.objects.get_or_create(name=cls.get_name())
        user.groups.add(group)
        permissions_to_add = cls.get_default_true_permissions()
        user.user_permissions.add(*permissions_to_add)

        return group

    @classmethod
    def _get_adjusted_true_permissions(cls, user):
        """
        Get all true permissions for a user excluding ones that
        have been explicitly revoked.
        """
        from rolepermissions.permissions import available_perm_status

        default_true_permissions = set()
        user_permission_states = available_perm_status(user)
        adjusted_true_permissions = set()

        # Grab the default true permissions from each of the user's roles
        for role in get_user_roles(user):
            default_true_permissions.update(role.get_default_true_permissions())

        # For each of those default true permissions, only keep ones
        # that haven't been explicitly revoked
        for permission in default_true_permissions:
            if user_permission_states[permission.codename]:
                adjusted_true_permissions.add(permission)

        return adjusted_true_permissions

    @classmethod
    def remove_role_from_user(cls, user):
        """
        Remove this role from a user.

        WARNING: Any permissions that were explicitly granted to the user
        that are also defined to be granted by this role will be revoked
        when this role is revoked.

        Example:
            >>> class Doctor(AbstractUserRole):
            ...     available_permissions = {
            ...         "operate": False,
            ...     }
            >>>
            >>> class Surgeon(AbstractUserRole):
            ...     available_permissions = {
            ...         "operate": True,
            ...     }
            >>>
            >>> grant_permission(user, "operate")
            >>> remove_role(user, Surgeon)
            >>>
            >>> has_permission(user, "operate")
            False
            >>>

        In the example, the user no longer has the ``"operate"`` permission,
        even though it was set explicitly before the ``Surgeon`` role was removed.
        """

        # Grab the adjusted true permissions before the removal
        current_adjusted_true_permissions = cls._get_adjusted_true_permissions(user)

        group, _created = cls.get_or_create_group()
        user.groups.remove(group)

        # Grab the adjusted true permissions after the removal
        new_adjusted_true_permissions = cls._get_adjusted_true_permissions(user)

        # Remove true permissions that were default granted only by the removed role
        permissions_to_remove = (current_adjusted_true_permissions
                                 .difference(new_adjusted_true_permissions))
        for permission in permissions_to_remove:
            user.user_permissions.remove(permission)

        return group

    @classmethod
    def permission_names_list(cls):
        available_permissions = getattr(cls, 'available_permissions', {})
        return available_permissions.keys()

    @classmethod
    def get_all_permissions(cls):
        permission_names = list(cls.permission_names_list())
        if permission_names:
            return cls.get_or_create_permissions(permission_names)

        return []

    @classmethod
    def get_default_true_permissions(cls):
        if hasattr(cls, 'available_permissions'):
            permission_names = [
                key for (key, default) in
                cls.available_permissions.items() if default]

            return cls.get_or_create_permissions(permission_names)

        return []

    @classmethod
    def get_or_create_permissions(cls, permission_names):
        user_ct = ContentType.objects.get_for_model(get_user_model())
        permissions = list(Permission.objects.filter(
            content_type=user_ct, codename__in=permission_names).all())

        missing_permissions = set(permission_names) - set((p.codename for p in permissions))
        if len(missing_permissions) > 0:
            for permission_name in missing_permissions:
                permission, created = get_or_create_permission(permission_name)
                if created:  # assert created is True
                    permissions.append(permission)

        return permissions

    @classmethod
    def get_default(cls, permission_name):
        return cls.available_permissions[permission_name]

    @classmethod
    def get_or_create_group(cls):
        return Group.objects.get_or_create(name=cls.get_name())


def get_or_create_permission(codename, name=camel_or_snake_to_title):
    """
    Get a Permission object from a permission name.
    @:param codename: permission code name
    @:param name: human-readable permissions name (str) or callable that takes codename as
                  argument and returns str
    """
    user_ct = ContentType.objects.get_for_model(get_user_model())
    return Permission.objects.get_or_create(content_type=user_ct, codename=codename,
                                            defaults={'name': name(codename) if callable(name) else name})


def retrieve_role(role_name):
    """Get a Role object from a role name."""
    return RolesManager.retrieve_role(role_name)


def get_user_roles(user):
    """Get a list of a users's roles."""
    if user:
        groups = user.groups.all()   # Important! all() query may be cached on User with prefetch_related.
        roles = (RolesManager.retrieve_role(group.name) for group in groups if group.name in RolesManager.get_roles_names())
        return sorted(roles, key=lambda r: r.get_name() )
    else:
        return []


def _assign_or_remove_role(user, role, method_name):
    role_cls = role
    if not inspect.isclass(role):
        role_cls = retrieve_role(role)

    if not role_cls:
        raise RoleDoesNotExist

    getattr(role_cls, method_name)(user)

    return role_cls


def assign_role(user, role):
    """Assign a role to a user."""
    return _assign_or_remove_role(user, role, "assign_role_to_user")


def remove_role(user, role):
    """Remove a role from a user."""
    return _assign_or_remove_role(user, role, "remove_role_from_user")


def clear_roles(user):
    """Remove all roles from a user."""
    roles = get_user_roles(user)

    for role in roles:
        role.remove_role_from_user(user)

    return roles



from collections import namedtuple
from django.core.management import call_command
from django.test import TestCase

try:
    # django 3 don't have django.utils.six
    from django.utils.six import StringIO
except ImportError:
    from io import StringIO

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission

from model_mommy import mommy

from rolepermissions.roles import AbstractUserRole, get_user_roles
from rolepermissions.admin import RolePermissionsUserAdminMixin


class AdminRole1(AbstractUserRole):
    available_permissions = {
        'admin_perm1': True,
        'admin_perm2': False,
    }


class UserAdminMixinTest(TestCase):

    class UserAdminMock:
        def save_related(self, request, form, formsets, change):
            pass

    class CustomUserAdminMock(RolePermissionsUserAdminMixin, UserAdminMock):
        pass

    FormMock = namedtuple('FormMock', ['instance', ])

    def setup(self):
        pass

    def test_admin_save_related_syncs_roles(self):
        user = mommy.make(get_user_model())
        grp1 = mommy.make(Group)
        grp2 = mommy.make(Group, name=AdminRole1.get_name())
        user.groups.add(grp1)
        user.groups.add(grp2)
        form = self.FormMock(instance=user)
        self.CustomUserAdminMock().save_related(None, form, None, None)
        user_roles = get_user_roles(user)
        self.assertNotIn(grp1.name, (role.get_name() for role in user_roles))
        self.assertIn(AdminRole1, user_roles)


class SyncRolesTest(TestCase):

    def setup(self):
        pass

    def test_sync_group(self):
        out = StringIO()
        call_command('sync_roles', stdout=out)
        self.assertIn('Created Group: %s' % AdminRole1.get_name(), out.getvalue())
        group_names = [group['name'] for group in Group.objects.all().values('name')]
        self.assertIn(AdminRole1.get_name(), group_names)

    def test_sync_permissions(self):
        out = StringIO()
        call_command('sync_roles', stdout=out)
        permissions = [perm['codename'] for perm in Permission.objects.all().values('codename')]
        self.assertIn('admin_perm1', permissions)
        self.assertNotIn('admin_perm2', permissions)

    def test_sync_all_permissions(self):
        out = StringIO()
        call_command('sync_roles', all_permissions=True, stdout=out)
        permissions = [perm['codename'] for perm in Permission.objects.all().values('codename')]
        self.assertIn('admin_perm1', permissions)
        self.assertIn('admin_perm2', permissions)

    def test_sync_user_role_permissions(self):
        user = mommy.make(get_user_model())
        grp1 = mommy.make(Group)
        grp2 = mommy.make(Group, name=AdminRole1.get_name())
        user.groups.add(grp1)
        user.groups.add(grp2)
        out = StringIO()
        call_command('sync_roles', reset_user_permissions=True, stdout=out)

        user_group_names = [group['name'] for group in user.groups.all().values('name')]
        self.assertIn(grp1.name, user_group_names)
        self.assertIn(grp2.name, user_group_names)

        user_permission_names = [perm['codename'] for perm in user.user_permissions.all().values('codename')]
        self.assertIn('admin_perm1', user_permission_names)
        self.assertNotIn('admin_perm2', user_permission_names)

    def test_sync_preserves_groups(self):
        grp1 = mommy.make(Group)
        grp2 = mommy.make(Group, name=AdminRole1.get_name())
        out = StringIO()
        call_command('sync_roles', stdout=out)
        group_names = [group['name'] for group in Group.objects.all().values('name')]
        self.assertIn(grp1.name, group_names)
        self.assertIn(grp2.name, group_names)



from __future__ import unicode_literals

from rolepermissions.exceptions import (
    RolePermissionScopeException, CheckerNotRegistered)
from rolepermissions.roles import get_user_roles, get_or_create_permission


class PermissionsManager(object):
    _checkers = {}

    @classmethod
    def register_checker(cls, name, function):
        cls._checkers[name] = function

    @classmethod
    def get_checkers(cls):
        return cls._checkers

    @classmethod
    def retrieve_checker(cls, checker_name):
        if checker_name in cls._checkers:
            return cls._checkers[checker_name]

        raise CheckerNotRegistered('Checker with name %s was not registered' % checker_name)


def register_object_checker(name=None):
    def fuction_decorator(func):
        checker_name = name if name else func.__name__
        PermissionsManager.register_checker(checker_name, func)
    return fuction_decorator


def get_permission(permission_name):
    """Get a Permission object from a permission name."""
    permission, created = get_or_create_permission(permission_name)

    return permission


def available_perm_status(user):
    """
    Get a boolean map of the permissions available to a user
    based on that user's roles.
    """
    roles = get_user_roles(user)
    permission_hash = {}

    for role in roles:
        permission_names = role.permission_names_list()

        for permission_name in permission_names:
            permission_hash[permission_name] = get_permission(
                permission_name) in user.user_permissions.all()

    return permission_hash


def available_perm_names(user):
    """
    Return a list of permissions codenames available to a user, based on that user's roles.
      i.e., keys for all "True" permissions from available_perm_status(user):
       Assert: set(available_perm_names(user)) == set(perm for perm,has_perm in available_perm_status(user) if has_perm)
       Query efficient; especially when prefetch_related('group', 'user_permissions') on user object.
       No side-effects; permissions are not created in DB as side-effect.
    """
    roles = get_user_roles(user)
    perm_names = set(p for role in roles for p in role.permission_names_list())
    return [p.codename for p in user.user_permissions.all() if p.codename in perm_names] \
                                                                        if roles else []  # e.g., user == None


def grant_permission(user, permission_name):
    """
    Grant a user a specified permission.

    Permissions are only granted if they are in the scope any of the
    user's roles. If the permission is out of scope,
    a RolePermissionScopeException is raised.
    """
    roles = get_user_roles(user)

    for role in roles:
        if permission_name in role.permission_names_list():
            permission = get_permission(permission_name)
            user.user_permissions.add(permission)
            return

    raise RolePermissionScopeException(
        "This permission isn't in the scope of "
        "any of this user's roles.")


def revoke_permission(user, permission_name):
    """
    Revoke a specified permission from a user.

    Permissions are only revoked if they are in the scope any of the user's
    roles. If the permission is out of scope, a RolePermissionScopeException
    is raised.
    """
    roles = get_user_roles(user)

    for role in roles:
        if permission_name in role.permission_names_list():
            permission = get_permission(permission_name)
            user.user_permissions.remove(permission)
            return

    raise RolePermissionScopeException(
        "This permission isn't in the scope of "
        "any of this user's roles.")




from django.test import TestCase
from rolepermissions.utils import camelToSnake, snake_to_title, camel_or_snake_to_title


class UtilTests(TestCase):

    def setUp(self):
        pass

    def test_camel_to_snake(self):
        self.assertEqual(camelToSnake('camelCaseString'), 'camel_case_string')
        self.assertEqual(camelToSnake('Snake_Camel_String'), 'snake__camel__string')

    def test_snake_to_title(self):
        self.assertEqual(snake_to_title('snake_case_string'), 'Snake Case String')
        self.assertEqual(snake_to_title('Even_if__itsFunky'), 'Even If  Itsfunky')

    def test_camel_or_snake_to_title(self):
        self.assertEqual(camel_or_snake_to_title('snake_case_string'), 'Snake Case String')
        self.assertEqual(camel_or_snake_to_title('camelCaseString'), 'Camel Case String')
        self.assertEqual(camel_or_snake_to_title('mix_itUp_WhyNot'), 'Mix It Up  Why Not')



from django.conf import settings
from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model
from rolepermissions import roles


class Command(BaseCommand):
    ROLEPERMISSIONS_MODULE = getattr(settings, 'ROLEPERMISSIONS_MODULE', 'roles.py')
    help = "Synchronize auth Groups and Permissions with UserRoles defined in %s." % ROLEPERMISSIONS_MODULE
    version = "1.0.0"

    def get_version(self):
        return self.version

    def add_arguments(self, parser):
        # Optional argument
        parser.add_argument(
            '--reset_user_permissions',
            action='store_true',
            dest='reset_user_permissions',
            default=False,
            help='Re-assign all User roles -- resets user Permissions to defaults defined by role(s) !! CAUTION !!',
        )
        parser.add_argument(
            '--all_permissions',
            action='store_true',
            dest='all_permissions',
            default=False,
            help='Create all new permissions instead of only permissions with default true',
        )

    def handle(self, *args, **options):
        # Sync auth.Group with current registered roles (leaving existing groups intact!)
        for role in roles.RolesManager.get_roles():
            group, created = role.get_or_create_group()
            if created:
                self.stdout.write("Created Group: %s from Role: %s" % (group.name, role.get_name()))
            # Sync auth.Permission with permissions for this role
            if options.get('all_permissions', False):
                role.get_all_permissions()
            else:
                role.get_default_true_permissions()

        if options.get('reset_user_permissions', False):  # dj1.7 compat
            # Push any permission changes made to roles and remove any unregistered roles from all auth.Users
            self.stdout.write("Resetting permissions for ALL Users to defaults defined by roles.")

            for user in get_user_model().objects.all():
                user_roles = roles.get_user_roles(user=user)
                roles.clear_roles(user=user)
                for role in user_roles:
                    roles.assign_role(user=user, role=role)




from django.apps import AppConfig

from rolepermissions.loader import load_roles_and_permissions


class RolePermissions(AppConfig):
    name = 'rolepermissions'
    verbose_name = "Django Role Permissions"

    def ready(self):
        load_roles_and_permissions()




from django.test import TestCase
from django.contrib.auth import get_user_model

from model_mommy import mommy

from rolepermissions.roles import AbstractUserRole
from rolepermissions.checkers import has_role, has_permission, has_object_permission
from rolepermissions.permissions import register_object_checker


class VerRole1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }

class VerRole2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }

class VerRole3(AbstractUserRole):
    role_name = 'ver_new_name'
    available_permissions = {
        'permission5': False,
        'permission6': False,
    }


class HasRoleTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())

        VerRole1.assign_role_to_user(self.user)

    def test_user_has_VerRole1(self):
        user = self.user

        self.assertTrue(has_role(user, VerRole1))

    def test_user_does_not_have_VerRole2(self):
        user = self.user

        self.assertFalse(has_role(user, VerRole2))

    def test_user_has_VerRole1_or_VerRole2(self):
        user = self.user

        self.assertTrue(has_role(user, [VerRole1, VerRole2]))

    def test_has_role_by_name(self):
        user = self.user

        self.assertTrue(has_role(user, 'ver_role1'))

    def test_user_has_VerRole1_or_VerRole3_by_name(self):
        user = self.user

        VerRole3.assign_role_to_user(user)

        self.assertTrue(has_role(user, ['ver_role1', 'ver_new_name']))

    def test_not_existent_role(self):
        user = self.user

        self.assertFalse(has_role(user, 'not_a_role'))

    def test_none_user_param(self):
        self.assertFalse(has_role(None, 'ver_role1'))


class HasPermissionTests(TestCase):
    def setUp(self):
        self.user = mommy.make(get_user_model())

        VerRole1.assign_role_to_user(self.user)

    def test_has_VerRole1_permission(self):
        user = self.user

        self.assertTrue(has_permission(user, 'permission1'))

    def test_dos_not_have_VerRole1_permission(self):
        user = self.user

        VerRole1.assign_role_to_user(user)

        self.assertFalse(has_permission(user, 'permission3'))

    def test_not_existent_permission(self):
        user = self.user

        self.assertFalse(has_permission(user, 'not_a_permission'))

    def test_user_with_no_role(self):
        user = mommy.make(get_user_model())

        self.assertFalse(has_permission(user, 'permission1'))

    def test_none_user_param(self):
        self.assertFalse(has_permission(None, 'ver_role1'))

    def test_queries_no_prefetch(self):
        fetched_user = get_user_model().objects.get(pk=self.user.pk)
        N = 3
        with self.assertNumQueries(2 * N):  # Two query (fetch roles, fetch permissions) per call
            for i in range(N):
                has_permission(fetched_user, 'permission1')

    def test_queries_with_prefetch(self):
        fetched_user = get_user_model().objects.prefetch_related('groups', 'user_permissions').get(pk=self.user.pk)
        N = 3
        with self.assertNumQueries(0):  # all data required is cached with fetched_user
            for i in range(N):
                has_permission(fetched_user, 'permission1')


class HasObjectPermissionTests(TestCase):

    def setUp(self):
        self.user = mommy.make(get_user_model())

        VerRole1.assign_role_to_user(self.user)

        @register_object_checker()
        def obj_checker(role, user, obj):
            return obj and True

    def test_has_object_permission(self):
        user = self.user

        self.assertTrue(has_object_permission('obj_checker', user, True))

    def test_does_not_have_object_permission(self):
        user = self.user

        self.assertFalse(has_object_permission('obj_checker', user, False))

    def test_check_none_role_if_user_has_no_role(self):
        user = mommy.make(get_user_model())

        self.assertTrue(has_object_permission('obj_checker', user, True))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re
import os
import sys


package = 'rolepermissions'

requirements = [
    'Django>=1.5',
    'django-discover-runner',
    'six',
]
test_requirements = [
    'model-mommy==1.3.1',
    'mock'
]


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}

# python setup.py register
if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s'" % args)
    print("  git push --tags")
    sys.exit()


setup(
    name='django-role-permissions',
    version=get_version(package),
    url='http://github.com/vintasoftware/django-role-permissions',
    license='MIT',
    description='A django app for role based permissions.',
    author='Filipe Ximenes',
    author_email='filipeximenes@gmail.com',
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=requirements,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    test_suite='rolepermissions.tests',
    tests_require=test_requirements
)



from __future__ import unicode_literals

import inspect

from importlib import import_module
from pydoc import locate

from django.conf import settings


def get_app_name(app_name):
    """
    Returns a app name from new app config if is
    a class or the same app name if is not a class.
    """
    type_ = locate(app_name)
    if inspect.isclass(type_):
        return type_.name
    return app_name


def load_roles_and_permissions():
    if hasattr(settings, 'ROLEPERMISSIONS_MODULE'):
        import_module(settings.ROLEPERMISSIONS_MODULE)

    for app_name in settings.INSTALLED_APPS:
        if app_name != 'rolepermissions':
            app_name = get_app_name(app_name)
            try:
                import_module('.permissions', app_name)
            except ImportError:
                pass



from __future__ import unicode_literals

from rolepermissions.decorators import has_permission_decorator, has_role_decorator


class HasRoleMixin(object):
    allowed_roles = []
    redirect_to_login = None

    def dispatch(self, request, *args, **kwargs):
        roles = self.allowed_roles
        return (has_role_decorator(roles, redirect_to_login=self.redirect_to_login)
                (super(HasRoleMixin, self).dispatch)
                (request, *args, **kwargs))


class HasPermissionsMixin(object):
    required_permission = ''
    redirect_to_login = None

    def dispatch(self, request, *args, **kwargs):
        permission = self.required_permission
        return (has_permission_decorator(permission, redirect_to_login=self.redirect_to_login)
                (super(HasPermissionsMixin, self).dispatch)
                (request, *args, **kwargs))



from __future__ import unicode_literals

import inspect

from rolepermissions.roles import (
    RolesManager, get_user_roles)
from rolepermissions.permissions import (
    PermissionsManager, available_perm_names)


def has_role(user, roles):
    """Check if a user has any of the given roles."""
    if user and user.is_superuser:
        return True

    if not isinstance(roles, list):
        roles = [roles]

    normalized_roles = []
    for role in roles:
        if not inspect.isclass(role):
            role = RolesManager.retrieve_role(role)

        normalized_roles.append(role)

    user_roles = get_user_roles(user)

    return any([role in user_roles for role in normalized_roles])


def has_permission(user, permission_name):
    """Check if a user has a given permission."""
    if user and user.is_superuser:
        return True

    return permission_name in available_perm_names(user)

def has_object_permission(checker_name, user, obj):
    """Check if a user has permission to perform an action on an object."""
    if user and user.is_superuser:
        return True

    checker = PermissionsManager.retrieve_checker(checker_name)
    user_roles = get_user_roles(user)

    if not user_roles:
        user_roles = [None]

    return any([checker(user_role, user, obj) for user_role in user_roles])
